@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22097 $
+ * $Revision: 23729 $
  * $Author: rjongbloed $
- * $Date: 2009-02-21 04:55:10 +0000 (Sat, 21 Feb 2009) $
+ * $Date: 2009-10-29 23:46:33 +0000 (Thu, 29 Oct 2009) $
  */
 
 #ifndef OPAL_H323_H323CON_H
@@ -1747,8 +1747,11 @@ class H323Connection : public OpalRTPConnection
   /**@name Other services */
   //@{
     /**Request a mode change to T.38 data.
+       Note this function is strictly H.323 and does operate correctly in the
+       OPAL media stream model. It is maintained for backward compatibility
+       with older applications only.
       */
-    virtual PBoolean RequestModeChangeT38(
+    virtual PBoolean P_DEPRECATED RequestModeChangeT38(
       const char * capabilityNames = "T.38\nT38FaxUDP"
     );
 
@@ -1983,7 +1986,6 @@ class H323Connection : public OpalRTPConnection
     PString            gkAccessTokenOID;
     PBYTEArray         gkAccessTokenData;
     PBoolean               addAccessTokenToSetup;
-    PBoolean               alertDone;
 
     H323Transport * signallingChannel;
     H323Transport * controlChannel;
@@ -2045,7 +2047,6 @@ class H323Connection : public OpalRTPConnection
     OpalMediaStreamPtr     fastStartMediaStream;
     
     // H.245 - compliant handling of session IDs
-    PBoolean defaultAudioSessionIDAssigned, defaultVideoSessionIDAssigned;
     unsigned nextSessionID;
     typedef std::map<unsigned, unsigned> InternalToExternalSessionIDMap;
     typedef std::map<OpalMediaType, unsigned> MediaTypeToInternalSessionIDMap;
