@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21816 $
+ * $Revision: 22096 $
  * $Author: rjongbloed $
- * $Date: 2008-12-17 02:54:09 +0000 (Wed, 17 Dec 2008) $
+ * $Date: 2009-02-21 03:25:23 +0000 (Sat, 21 Feb 2009) $
  */
 
 #ifndef OPAL_OPAL_ENDPOINT_H
@@ -263,6 +263,24 @@ class OpalEndPoint : public PObject
       OpalConnection & connection   ///<  Connection that is calling
     );
 
+    /**Call back for remote party is now responsible for completing the call.
+       This function is called when the remote system has been contacted and it
+       has accepted responsibility for completing, or failing, the call. This
+       is distinct from OnAlerting() in that it is not known at this time if
+       anything is ringing. This indication may be used to distinguish between
+       "transport" level error, in which case another host may be tried, and
+       that finalising the call has moved "upstream" and the local system has
+       no more to do but await a result.
+
+       If an application overrides this function, it should generally call the
+       ancestor version for correct operation.
+
+       The default behaviour calls the OpalManager function of the same name.
+     */
+    virtual void OnProceeding(
+      OpalConnection & connection   ///<  Connection that is proceeeding
+    );
+
     /**Call back for remote party being alerted.
        This function is called after the connection is informed that the
        remote endpoint is "ringing". Generally some time after the
@@ -279,7 +297,7 @@ class OpalEndPoint : public PObject
        The default behaviour calls the OpalManager function of the same name.
      */
     virtual void OnAlerting(
-      OpalConnection & connection   ///<  Connection that was established
+      OpalConnection & connection   ///<  Connection that is alerting
     );
 
     /**Call back for answering an incoming call.
