@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23467 $
+ * $Revision: 23625 $
  * $Author: rjongbloed $
- * $Date: 2009-09-16 07:50:34 +0000 (Wed, 16 Sep 2009) $
+ * $Date: 2009-10-09 01:44:30 +0000 (Fri, 09 Oct 2009) $
  */
 
 #ifndef OPAL_SIP_SIPCON_H
@@ -420,6 +420,18 @@ class SIPConnection : public OpalRTPConnection
       const char * extra = NULL,
       const SDPSessionDescription * sdp = NULL
     );
+
+    enum TypeOfINVITE {
+      IsNewINVITE,
+      IsDuplicateINVITE,
+      IsReINVITE,
+      IsLoopedINVITE
+    };
+
+    /// Indicate if this is a duplicate or multi-path INVITE
+    TypeOfINVITE CheckINVITE(
+      const SIP_PDU & pdu
+    ) const;
   //@}
 
     OpalTransportAddress GetDefaultSDPConnectAddress(WORD port = 0) const;
@@ -429,7 +441,7 @@ class SIPConnection : public OpalRTPConnection
     SIPEndPoint & GetEndPoint() const { return endpoint; }
     SIPDialogContext & GetDialog() { return m_dialog; }
     const SIPDialogContext & GetDialog() const { return m_dialog; }
-    SIPAuthentication * GetAuthenticator() const { return authentication; }
+    SIPAuthentication * GetAuthenticator() const { return m_authentication; }
 
 #if OPAL_VIDEO
     /**Call when SIP INFO of type application/media_control+xml is received.
@@ -531,7 +543,8 @@ class SIPConnection : public OpalRTPConnection
     OpalGloballyUniqueID  m_dialogNotifyId;
     int                   m_appearanceCode;
     PString               m_alertInfo;
-    SIPAuthentication   * authentication;
+    SIPAuthentication   * m_authentication;
+    unsigned              m_authenticatedCseq;
 
     std::map<SIP_PDU::Methods, unsigned> m_lastRxCSeq;
 
