@@ -23,23 +23,29 @@
  * 2550 Garcia Avenue
  * Mountain View, California  94043
  */
-/* 16kbps version created, used 24kbps code and changing as little as possible.
- * G.726 specs are available from ITU's gopher or WWW site (http://www.itu.ch)
- * If any errors are found, please contact me at mrand@tamu.edu
- *      -Marc Randolph
- */
 
 /*
- * g726_16.c
+ * g726_32.c
  *
  * Description:
  *
- * g723_16_encoder(), g723_16_decoder()
+ * g721_encoder(), g721_decoder()
  *
- * These routines comprise an implementation of the CCITT G.726 16 Kbps
- * ADPCM coding algorithm.  Essentially, this implementation is identical to
- * the bit level description except for a few deviations which take advantage
- * of workstation attributes, such as hardware 2's complement arithmetic.
+ * These routines comprise an implementation of the CCITT G.721 ADPCM
+ * coding algorithm.  Essentially, this implementation is identical to
+ * the bit level description except for a few deviations which
+ * take advantage of work station attributes, such as hardware 2's
+ * complement arithmetic and large memory.  Specifically, certain time
+ * consuming operations such as multiplications are replaced
+ * with lookup tables and software 2's complement operations are
+ * replaced with hardware 2's complement.
+ *
+ * The deviation from the bit level specification (lookup tables)
+ * preserves the bit level performance specifications.
+ *
+ * As outlined in the G.721 Recommendation, the algorithm is broken
+ * down into modules.  Each section of code below is preceded by
+ * the name of the module which it is implementing.
  *
  * The ITU-T G.726 coder is an adaptive differential pulse code modulation
  * (ADPCM) waveform coding algorithm, suitable for coding of digitized
@@ -58,48 +64,39 @@
 #include "g72x.h"
 #include "private.h"
 
+static int qtab_721[7] = {-124, 80, 178, 246, 300, 349, 400};
 /*
- * Maps G.723_16 code word to reconstructed scale factor normalized log
- * magnitude values.  Comes from Table 11/G.726
+ * Maps G.721 code word to reconstructed scale factor normalized log
+ * magnitude values.
  */
-static short	_dqlntab[4] = { 116, 365, 365, 116}; 
-
-/* Maps G.723_16 code word to log of scale factor multiplier.
- *
- * _witab[4] is actually {-22 , 439, 439, -22}, but FILTD wants it
- * as WI << 5  (multiplied by 32), so we'll do that here 
- */
-static short	_witab[4] = {-704, 14048, 14048, -704};
+static short	_dqlntab[16] = {-2048, 4, 135, 213, 273, 323, 373, 425,
+				425, 373, 323, 273, 213, 135, 4, -2048};
 
+/* Maps G.721 code word to log of scale factor multiplier. */
+static short	_witab[16] = {-12, 18, 41, 64, 112, 198, 355, 1122,
+				1122, 355, 198, 112, 64, 41, 18, -12};
 /*
- * Maps G.723_16 code words to a set of values whose long and short
+ * Maps G.721 code words to a set of values whose long and short
  * term averages are computed and then compared to give an indication
  * how stationary (steady state) the signal is.
  */
-
-/* Comes from FUNCTF */
-static short	_fitab[4] = {0, 0xE00, 0xE00, 0};
-
-/* Comes from quantizer decision level tables (Table 7/G.726)
- */
-static int qtab_723_16[1] = {261};
-
+static short	_fitab[16] = {0, 0, 0, 0x200, 0x200, 0x200, 0x600, 0xE00,
+				0xE00, 0x600, 0x200, 0x200, 0x200, 0, 0, 0};
 
 /*
- * g723_16_encoder()
+ * g721_encoder()
  *
- * Encodes a linear PCM, A-law or u-law input sample and returns its 2-bit code.
- * Returns -1 if invalid input coding value.
+ * Encodes the input vale of linear PCM, A-law or u-law data sl and returns
+ * the resulting code. -1 is returned for unknown input coding value.
  */
 int
-g726_16_encoder(
+g726_32_encoder(
 	int		sl,
 	int		in_coding,
 	g726_state *state_ptr)
 {
 	int		sezi;
 	int		sez;			/* ACCUM */
-	int		sei;
 	int		se;
 	int		d;				/* SUBTA */
 	int		y;				/* MIX */
@@ -116,7 +113,7 @@ g726_16_encoder(
 		sl = ulaw2linear(sl) >> 2;
 		break;
 	case AUDIO_ENCODING_LINEAR:
-		sl >>= 2;		/* sl of 14-bit dynamic range */
+		sl >>= 2;			/* 14-bit dynamic range */
 		break;
 	default:
 		return (-1);
@@ -124,42 +121,36 @@ g726_16_encoder(
 
 	sezi = predictor_zero(state_ptr);
 	sez = sezi >> 1;
-	sei = sezi + predictor_pole(state_ptr);
-	se = sei >> 1;			/* se = estimated signal */
+	se = (sezi + predictor_pole(state_ptr)) >> 1;	/* estimated signal */
 
-	d = sl - se;			/* d = estimation diff. */
+	d = sl - se;				/* estimation difference */
 
-	/* quantize prediction difference d */
-	y = step_size(state_ptr);	/* quantizer step size */
-	i = quantize(d, y, qtab_723_16, 1);  /* i = ADPCM code */
+	/* quantize the prediction difference */
+	y = step_size(state_ptr);		/* quantizer step size */
+	i = quantize(d, y, qtab_721, 7);	/* i = ADPCM code */
 
-	      /* Since quantize() only produces a three level output
-	       * (1, 2, or 3), we must create the fourth one on our own
-	       */
-	if (i == 3)                          /* i code for the zero region */
-	  if ((d & 0x8000) == 0)             /* If d > 0, i=3 isn't right... */
-	    i = 0;
-	    
-	dq = reconstruct(i & 2, _dqlntab[i], y); /* quantized diff. */
+	dq = reconstruct(i & 8, _dqlntab[i], y);	/* quantized est diff */
 
-	sr = (dq < 0) ? se - (dq & 0x3FFF) : se + dq; /* reconstructed signal */
+	sr = (dq < 0) ? se - (dq & 0x3FFF) : se + dq;	/* reconst. signal */
 
-	dqsez = sr + sez - se;		/* pole prediction diff. */
+	dqsez = sr + sez - se;			/* pole prediction diff. */
 
-	update(2, y, _witab[i], _fitab[i], dq, sr, dqsez, state_ptr);
+	update(4, y, _witab[i] << 5, _fitab[i], dq, sr, dqsez, state_ptr);
 
 	return (i);
 }
 
 /*
- * g723_16_decoder()
+ * g721_decoder()
  *
- * Decodes a 2-bit CCITT G.723_16 ADPCM code and returns
- * the resulting 16-bit linear PCM, A-law or u-law sample value.
- * -1 is returned if the output coding is unknown.
+ * Description:
+ *
+ * Decodes a 4-bit code of G.721 encoded data of i and
+ * returns the resulting linear PCM, A-law or u-law value.
+ * return -1 for unknown out_coding value.
  */
 int
-g726_16_decoder(
+g726_32_decoder(
 	int		i,
 	int		out_coding,
 	g726_state *state_ptr)
@@ -172,29 +163,34 @@ g726_16_decoder(
 	int		dq;
 	int		sr;				/* ADDB */
 	int		dqsez;
+	long	lino;
 
-	i &= 0x03;			/* mask to get proper bits */
+	i &= 0x0f;				/* mask to get proper bits */
 	sezi = predictor_zero(state_ptr);
 	sez = sezi >> 1;
 	sei = sezi + predictor_pole(state_ptr);
 	se = sei >> 1;			/* se = estimated signal */
 
-	y = step_size(state_ptr);	/* adaptive quantizer step size */
-	dq = reconstruct(i & 0x02, _dqlntab[i], y); /* unquantize pred diff */
+	y = step_size(state_ptr);	/* dynamic quantizer step size */
 
-	sr = (dq < 0) ? (se - (dq & 0x3FFF)) : (se + dq); /* reconst. signal */
+	dq = reconstruct(i & 0x08, _dqlntab[i], y); /* quantized diff. */
+
+	sr = (dq < 0) ? (se - (dq & 0x3FFF)) : se + dq;	/* reconst. signal */
 
 	dqsez = sr - se + sez;			/* pole prediction diff. */
 
-	update(2, y, _witab[i], _fitab[i], dq, sr, dqsez, state_ptr);
+	update(4, y, _witab[i] << 5, _fitab[i], dq, sr, dqsez, state_ptr);
 
 	switch (out_coding) {
 	case AUDIO_ENCODING_ALAW:
-		return (tandem_adjust_alaw(sr, se, y, i, 2, qtab_723_16));
+		return (tandem_adjust_alaw(sr, se, y, i, 8, qtab_721));
 	case AUDIO_ENCODING_ULAW:
-		return (tandem_adjust_ulaw(sr, se, y, i, 2, qtab_723_16));
+		return (tandem_adjust_ulaw(sr, se, y, i, 8, qtab_721));
 	case AUDIO_ENCODING_LINEAR:
-		return (sr << 2);	/* sr was of 14-bit dynamic range */
+        lino = (long)sr << 2;  /* this seems to overflow a short*/
+		lino = lino > 32767 ? 32767 : lino;
+		lino = lino < -32768 ? -32768 : lino;
+		return lino;//(sr << 2);	/* sr was 14-bit dynamic range */
 	default:
 		return (-1);
 	}
