@@ -19,9 +19,9 @@
  *
  * Contributor(s): Equivalence Pty. Ltd.
  *
- * $Revision: 22021 $
+ * $Revision: 22820 $
  * $Author: rjongbloed $
- * $Date: 2009-02-08 04:50:19 +0000 (Sun, 08 Feb 2009) $
+ * $Date: 2009-06-05 03:17:24 +0000 (Fri, 05 Jun 2009) $
  */
 
 #include "precompile.h"
@@ -595,8 +595,9 @@ void CallThread::Main()
 
 void CallThread::Stop()
 {
-  if (!IsTerminated())
+  if (!IsTerminated()) {
     OUTPUT(index, PString::Empty(), "Stopping.");
+  }
 
   exit.Signal();
 }
