@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22073 $
+ * $Revision: 23165 $
  * $Author: rjongbloed $
- * $Date: 2009-02-16 11:21:01 +0000 (Mon, 16 Feb 2009) $
+ * $Date: 2009-07-29 04:57:29 +0000 (Wed, 29 Jul 2009) $
  */
 
 #ifdef __GNUC__
@@ -866,14 +866,14 @@ PBoolean OpalPluginFramedAudioTranscoder::ConvertSilentFrame(BYTE * buffer)
 //
 
 OpalPluginStreamedAudioTranscoder::OpalPluginStreamedAudioTranscoder(const PluginCodec_Definition * codecDefn,
-                                                                     bool isEncoder,
-                                                                     unsigned inputBits,
-                                                                     unsigned outputBits)
+                                                                     bool isEncoder)
   : OpalStreamedTranscoder(GetRawPCM(codecDefn->sourceFormat, codecDefn->sampleRate),
                            GetRawPCM(codecDefn->destFormat,   codecDefn->sampleRate),
-                           inputBits, outputBits)
+                           16, 16)
   , OpalPluginTranscoder(codecDefn, isEncoder)
-{ 
+{
+  (isEncoder ? outputBitsPerSample : inputBitsPerSample) =
+                  (codecDefn->flags & PluginCodec_BitsPerSampleMask) >> PluginCodec_BitsPerSamplePos;
   comfortNoise       = (codecDef->flags & PluginCodec_ComfortNoiseMask) == PluginCodec_ComfortNoise;
   acceptEmptyPayload = (codecDef->flags & PluginCodec_ComfortNoiseMask) == PluginCodec_EmptyPayload;
   acceptOtherPayloads = (codecDef->flags & PluginCodec_OtherPayloadMask) == PluginCodec_OtherPayload;
@@ -884,39 +884,13 @@ PBoolean OpalPluginStreamedAudioTranscoder::UpdateMediaFormats(const OpalMediaFo
   return OpalStreamedTranscoder::UpdateMediaFormats(input, output) && UpdateOptions(isEncoder ? output : input);
 }
 
-
-OpalPluginStreamedAudioEncoder::OpalPluginStreamedAudioEncoder(const PluginCodec_Definition * codecDefn, bool)
-  : OpalPluginStreamedAudioTranscoder(codecDefn, true,
-                                      16,
-                                      (codecDefn->flags & PluginCodec_BitsPerSampleMask) >> PluginCodec_BitsPerSamplePos)
+int OpalPluginStreamedAudioTranscoder::ConvertOne(int from) const
 {
-}
-
-int OpalPluginStreamedAudioEncoder::ConvertOne(int _sample) const
-{
-  short sample = (short)_sample;
-  unsigned int fromLen = sizeof(sample);
+  unsigned int fromLen = sizeof(from);
   int to;
   unsigned toLen = sizeof(to);
   unsigned flags = 0;
-  return Transcode(&sample, &fromLen, &to, &toLen, &flags) ? to : -1;
-}
-
-
-OpalPluginStreamedAudioDecoder::OpalPluginStreamedAudioDecoder(const PluginCodec_Definition * codecDefn, bool)
-  : OpalPluginStreamedAudioTranscoder(codecDefn, false,
-                                      (codecDefn->flags & PluginCodec_BitsPerSampleMask) >> PluginCodec_BitsPerSamplePos,
-                                      16)
-{
-}
-
-int OpalPluginStreamedAudioDecoder::ConvertOne(int codedSample) const
-{
-  unsigned int fromLen = sizeof(codedSample);
-  short to;
-  unsigned toLen = sizeof(to);
-  unsigned flags = 0;
-  return Transcode(&codedSample, &fromLen, &to, &toLen, &flags) ? to : -1;
+  return Transcode(&from, &fromLen, &to, &toLen, &flags) ? to : -1;
 }
 
 
@@ -928,6 +902,7 @@ OpalPluginVideoTranscoder::OpalPluginVideoTranscoder(const PluginCodec_Definitio
   : OpalVideoTranscoder(codecDefn->sourceFormat, codecDefn->destFormat)
   , OpalPluginTranscoder(codecDefn, isEncoder)
   , m_bufferRTP(NULL)
+  , m_lastVideoFastUpdate(PTimer::Tick())
 #if PTRACING
   , m_consecutiveIntraFrames(0)
 #endif
@@ -980,33 +955,34 @@ PBoolean OpalPluginVideoTranscoder::ConvertFrames(const RTP_DataFrame & src, RTP
         return false;
       }
 
-      if ((flags & PluginCodec_ReturnCoderIFrame) != 0)
-        lastFrameWasIFrame = true;
+      lastFrameWasIFrame = (flags & PluginCodec_ReturnCoderIFrame) != 0;
 
-      if (toLen > 0) {
+      if ((toLen >= RTP_DataFrame::MinHeaderSize) && ((PINDEX)toLen >= dst->GetHeaderSize())) {
         dst->SetPayloadSize(toLen - dst->GetHeaderSize());
         dstList.Append(dst);
       }
+      else
+        delete dst;
 
     } while ((flags & PluginCodec_ReturnCoderLastFrame) == 0);
     PTRACE(5, "OpalPlugin\tEncoded video frame into " << dstList.GetSize() << " packets.");
 
-#if OPAL_STATISTICS
     m_totalFrames++;
     if (lastFrameWasIFrame)
       m_keyFrames++;
-#endif
 
 #if PTRACING
     if (!lastFrameWasIFrame)
       m_consecutiveIntraFrames = 0;
     else if (forceIFrame)
-      PTRACE(3, "OpalPlugin\tSending I-Frame in response to videoFastUpdate");
-    else if (m_consecutiveIntraFrames < 10) {
-      if (++m_consecutiveIntraFrames >= 10)
-        PTRACE(3, "OpalPlugin\tSending many consecutive I-Frames, assuming codec cannot do P-Frames");
+      PTRACE(3, "OpalPlugin\tEncoder sent forced I-Frame at " << m_totalFrames);
+    else {
+      if (m_consecutiveIntraFrames == 0) 
+        PTRACE(4, "OpalPlugin\tEncoder sending I-Frame at " << m_totalFrames);
       else
-        PTRACE(4, "OpalPlugin\tSending I-Frame");
+        PTRACE(4, "OpalPlugin\tEncoder sending " << ((m_consecutiveIntraFrames == 0) ? "" : "consecutive ") << "I-Frame at " << m_totalFrames);
+      if (++m_consecutiveIntraFrames >= 10)
+        PTRACE(3, "OpalPlugin\tEncoder has sent too many consecutive I-Frames - assume codec cannot do P-Frames");
     }
 #endif
 
@@ -1035,7 +1011,7 @@ PBoolean OpalPluginVideoTranscoder::ConvertFrames(const RTP_DataFrame & src, RTP
     if (!Transcode((const BYTE *)src, &fromLen, m_bufferRTP->GetPointer(), &toLen, &flags))
       return false;
 
-    if ((flags & PluginCodec_ReturnCoderRequestIFrame) != 0 && commandNotifier != PNotifier()) {
+    if ((commandNotifier != PNotifier()) && (flags & PluginCodec_ReturnCoderRequestIFrame) != 0) {
       PTimeInterval tick = PTimer::Tick();
       // Don't send lots of consecutive VideoFastUpdate commands
       if (tick - m_lastVideoFastUpdate < 2000)
@@ -1048,18 +1024,32 @@ PBoolean OpalPluginVideoTranscoder::ConvertFrames(const RTP_DataFrame & src, RTP
       }
     }
 
-    if (toLen > (unsigned)m_bufferRTP->GetHeaderSize() && (flags & PluginCodec_ReturnCoderLastFrame) != 0) {
-      m_bufferRTP->SetPayloadSize(toLen);
-      dstList.Append(m_bufferRTP);
-      m_bufferRTP = NULL;
+    if (toLen > RTP_DataFrame::MinHeaderSize && (flags & PluginCodec_ReturnCoderLastFrame) != 0) {
+      // Do sanity check on returned data.
+      PINDEX headerSize = m_bufferRTP->GetHeaderSize();
+      if (PAssert(toLen > (unsigned)(headerSize+sizeof(OpalVideoTranscoder::FrameHeader)),
+                  "Invalid return size from video plug in")) {
+        toLen -= headerSize;
+        OpalVideoTranscoder::FrameHeader * videoHeader = (OpalVideoTranscoder::FrameHeader *)m_bufferRTP->GetPayloadPtr();
+        if (PAssert(videoHeader->x == 0 && videoHeader->y == 0 &&
+                    videoHeader->width < 10000 && videoHeader->height < 10000 &&
+                    toLen >= (videoHeader->width*videoHeader->height*3/2+sizeof(OpalVideoTranscoder::FrameHeader)),
+                    "Invalid frame returned from video plug in")) {
+          m_bufferRTP->SetPayloadSize(toLen);
+          m_bufferRTP->SetPayloadType(GetPayloadType(false));
+          dstList.Append(m_bufferRTP);
+          m_bufferRTP = NULL;
 
-      lastFrameWasIFrame = (flags & PluginCodec_ReturnCoderIFrame) != 0;
+          lastFrameWasIFrame = (flags & PluginCodec_ReturnCoderIFrame) != 0;
+          if (lastFrameWasIFrame) {
+            PTRACE(5, "OpalPlugin\tVideo decoder returned I-frame");
+          }
 
-#if OPAL_STATISTICS
-      m_totalFrames++;
-      if (lastFrameWasIFrame)
-        m_keyFrames++;
-#endif
+          m_totalFrames++;
+          if (lastFrameWasIFrame)
+            m_keyFrames++;
+        }
+      }
     }
   }
 
@@ -1478,7 +1468,9 @@ void OpalPluginCodecManager::RegisterCodecPlugins(unsigned int count, const Plug
   GetOpalPCM16_16KHZ();
   GetOpalPCM16_32KHZ();
   GetOpalPCM16_48KHZ();
+#if OPAL_VIDEO
   GetOpalYUV420P();
+#endif
 
   for (unsigned  i = 0; i < count; i++,codecDefn++) {
 #if PTRACING
@@ -1587,7 +1579,7 @@ void OpalPluginCodecManager::RegisterCodecPlugins(unsigned int count, const Plug
         new OpalPluginTranscoderFactory<OpalPluginFramedAudioTranscoder>::Worker(OpalTranscoderKey(src, dst), codecDefn, IsEncoder(*codecDefn));
         break;
       case PluginCodec_MediaTypeAudioStreamed:
-        new OpalPluginTranscoderFactory<OpalPluginStreamedAudioEncoder>::Worker(OpalTranscoderKey(src, dst), codecDefn, IsEncoder(*codecDefn));
+        new OpalPluginTranscoderFactory<OpalPluginStreamedAudioTranscoder>::Worker(OpalTranscoderKey(src, dst), codecDefn, IsEncoder(*codecDefn));
         break;
   #if OPAL_T38_CAPABILITY
       case PluginCodec_MediaTypeFax:
@@ -1620,11 +1612,10 @@ void OpalPluginCodecManager::RegisterCapability(const PluginCodec_Definition * c
 
   OpalPluginControl isValid(codecDefn, PLUGINCODEC_CONTROL_VALID_FOR_PROTOCOL);
   if (isValid.Exists() && !isValid.Call((void *)"h323", sizeof(const char *))) {
-    PTRACE(4, "OpalPlugin\tDeferring creation of H.323 capability for plugin codec " << codecDefn->descr);
+    PTRACE(2, "OpalPlugin\tNot adding H.323 capability for plugin codec " << codecDefn->descr << " as this has been specifically disabled");
     return;
   }
 
-    PTRACE(2, "OpalPlugin\tNot adding H.323 capability for plugin codec " << codecDefn->descr << " as this has been specifically disabled");
   // add the capability
   H323CodecPluginCapabilityMapEntry * map = NULL;
 
