@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21303 $
+ * $Revision: 23601 $
  * $Author: rjongbloed $
- * $Date: 2008-10-13 05:39:40 +0000 (Mon, 13 Oct 2008) $
+ * $Date: 2009-10-07 07:53:21 +0000 (Wed, 07 Oct 2009) $
  */
 
 #include <ptlib.h>
@@ -102,26 +102,9 @@ void OpalEndPoint::ShutDown()
 
   // Shut down the listeners as soon as possible to avoid race conditions
   listeners.RemoveAll();
-
-#if OPAL_RTP_AGGREGATE
-  // delete aggregators
-  {
-    PWaitAndSignal m(rtpAggregationMutex);
-    if (rtpAggregator != NULL) {
-      delete rtpAggregator;
-      rtpAggregator = NULL;
-    }
-  }
-#endif
 }
 
 
-PBoolean OpalEndPoint::MakeConnection(OpalCall & /*call*/, const PString & /*party*/, void * /*userData*/, unsigned int /*options*/, OpalConnection::StringOptions * /*stringOptions*/)
-{
-  PAssertAlways("Must implement descendant of OpalEndPoint::MakeConnection");
-  return PFalse;
-}
-
 void OpalEndPoint::PrintOn(ostream & strm) const
 {
   strm << "EP<" << prefixName << '>';
@@ -240,7 +223,7 @@ PStringArray OpalEndPoint::GetDefaultListeners() const
 OpalListener * OpalEndPoint::FindListener(const OpalTransportAddress & iface)
 {
   for (OpalListenerList::iterator listener = listeners.begin(); listener != listeners.end(); ++listener) {
-    if (listener->GetTransportAddress().IsEquivalent(iface))
+    if (listener->GetLocalAddress().IsEquivalent(iface, true))
       return &*listener;
   }
   return NULL;
@@ -271,8 +254,10 @@ static void AddTransportAddress(OpalTransportAddressArray & interfaceAddresses,
                                 WORD port,
                                 const PString & proto)
 {
-  PIPSocket::Address localIP = ip == natInterfaceIP ? natExternalIP : ip;
-  OpalTransportAddress addr(localIP, port, proto);
+  if (ip != natExternalIP && (natInterfaceIP.IsAny() || ip == natInterfaceIP))
+    AddTransportAddress(interfaceAddresses, natInterfaceIP, natExternalIP, natExternalIP, port, proto);
+
+  OpalTransportAddress addr(ip, port, proto);
   if (interfaceAddresses.GetValuesIndex(addr) == P_MAX_INDEX)
     interfaceAddresses.Append(new OpalTransportAddress(addr));
 }
@@ -282,23 +267,39 @@ static void AddTransportAddresses(OpalTransportAddressArray & interfaceAddresses
                                   bool excludeLocalHost,
                                   const PIPSocket::Address & natInterfaceIP,
                                   const PIPSocket::Address & natExternalIP,
+                                  const OpalTransportAddress & preferredAddress,
                                   const OpalTransportAddress & localAddress)
 {
-  PIPSocket::InterfaceTable interfaces;
+  if (!preferredAddress.IsEmpty() && !preferredAddress.IsEquivalent(localAddress, true))
+    return;
 
-  PString proto = localAddress.Left(localAddress.Find('$')+1);
-
-  PIPSocket::Address ip;
+  PIPSocket::Address localIP;
   WORD port = 0;
-  if (!localAddress.GetIpAndPort(ip, port) || !ip.IsAny() || !PIPSocket::GetInterfaceTable(interfaces))
-    AddTransportAddress(interfaceAddresses, natInterfaceIP, natExternalIP, ip, port, proto);
-  else {
+  if (!localAddress.GetIpAndPort(localIP, port))
+    return;
+
+  PCaselessString proto = localAddress.GetProto();
+  PIPSocket::InterfaceTable interfaces;
+  if (!localIP.IsAny() || !PIPSocket::GetInterfaceTable(interfaces)) {
+    AddTransportAddress(interfaceAddresses, natInterfaceIP, natExternalIP, localIP, port, proto);
+    return;
+  }
+
+
+  PIPSocket::Address preferredIP;
+  if (preferredAddress.GetIpAddress(preferredIP)) {
     for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
       PIPSocket::Address ip = interfaces[i].GetAddress();
-      if (!excludeLocalHost || !ip.IsLoopback())
+      if (ip == preferredIP)
         AddTransportAddress(interfaceAddresses, natInterfaceIP, natExternalIP, ip, port, proto);
     }
   }
+
+  for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
+    PIPSocket::Address ip = interfaces[i].GetAddress();
+    if (!excludeLocalHost || !ip.IsLoopback())
+      AddTransportAddress(interfaceAddresses, natInterfaceIP, natExternalIP, ip, port, proto);
+  }
 }
 
 
@@ -307,24 +308,50 @@ OpalTransportAddressArray OpalEndPoint::GetInterfaceAddresses(PBoolean excludeLo
 {
   OpalTransportAddressArray interfaceAddresses;
 
-  PIPSocket::Address natInterfaceIP = PIPSocket::GetDefaultIpAny();
+  OpalTransportAddress associatedLocalAddress, associatedRemoteAddress;
+  PIPSocket::Address natInterfaceIP;
   PIPSocket::Address natExternalIP;
   if (associatedTransport != NULL) {
+    associatedLocalAddress = associatedTransport->GetLocalAddress();
+    associatedRemoteAddress = associatedTransport->GetRemoteAddress();
+
     PIPSocket::Address remoteIP;
-    associatedTransport->GetRemoteAddress().GetIpAddress(remoteIP);
+    associatedRemoteAddress.GetIpAddress(remoteIP);
 
     PNatMethod * natMethod = manager.GetNatMethod(remoteIP);
     if (natMethod != NULL) {
       natMethod->GetInterfaceAddress(natInterfaceIP);
       natMethod->GetExternalAddress(natExternalIP);
     }
-
-    AddTransportAddresses(interfaceAddresses, false, natInterfaceIP, natExternalIP, associatedTransport->GetLocalAddress());
+    else if (manager.GetTranslationAddress().IsValid()) {
+      natInterfaceIP = PIPSocket::GetDefaultIpAny();
+      natExternalIP = manager.GetTranslationAddress();
+    }
   }
 
-  for (OpalListenerList::iterator listener = listeners.begin(); listener != listeners.end(); ++listener)
-    AddTransportAddresses(interfaceAddresses, excludeLocalHost, natInterfaceIP, natExternalIP, listener->GetTransportAddress());
+  OpalListenerList::iterator listener;
 
+  if (!associatedLocalAddress.IsEmpty()) {
+    for (listener = listeners.begin(); listener != listeners.end(); ++listener)
+      AddTransportAddresses(interfaceAddresses,
+                            excludeLocalHost,
+                            natInterfaceIP,
+                            natExternalIP,
+                            associatedLocalAddress,
+                            listener->GetLocalAddress(associatedRemoteAddress));
+  }
+
+  for (listener = listeners.begin(); listener != listeners.end(); ++listener)
+    AddTransportAddresses(interfaceAddresses,
+                          excludeLocalHost,
+                          natInterfaceIP,
+                          natExternalIP,
+                          OpalTransportAddress(),
+                          listener->GetLocalAddress());
+
+  PTRACE(4, "OpalMan\tListener interfaces: associated transport="
+         << (associatedTransport != NULL ? (const char *)associatedLocalAddress : "None")
+         << "\n    " << setfill(',') << interfaceAddresses);
   return interfaceAddresses;
 }
 
@@ -370,6 +397,8 @@ PBoolean OpalEndPoint::AddConnection(OpalConnection * connection)
   if (connection == NULL)
     return PFalse;
 
+  connection->SetStringOptions(m_defaultStringOptions, false);
+
   OnNewConnection(connection->GetCall(), *connection);
 
   connectionsActive.SetAt(connection->GetToken(), connection);
@@ -411,6 +440,11 @@ PBoolean OpalEndPoint::OnIncomingConnection(OpalConnection & connection, unsigne
 }
 
 
+void OpalEndPoint::OnProceeding(OpalConnection & connection)
+{
+  manager.OnProceeding(connection);
+}
+
 void OpalEndPoint::OnAlerting(OpalConnection & connection)
 {
   manager.OnAlerting(connection);
@@ -502,6 +536,8 @@ OpalMediaFormatList OpalEndPoint::GetMediaFormats() const
 
   // Sound cards can only do 16 bit PCM, but at various sample rates
   // The following will be in order of preference, so lets do wideband first
+  formats += OpalPCM16_48KHZ;
+  formats += OpalPCM16_32KHZ;
   formats += OpalPCM16_16KHZ;
   formats += OpalPCM16;
 
@@ -509,6 +545,8 @@ OpalMediaFormatList OpalEndPoint::GetMediaFormats() const
   AddVideoMediaFormats(formats);
 #endif
 
+  AddIMMediaFormats(formats);
+
   return formats;
 }
 
@@ -596,6 +634,16 @@ void OpalEndPoint::OnMWIReceived(const PString & party,
   manager.OnMWIReceived(party, type, extraInfo);
 }
 
+bool OpalEndPoint::FindListenerForProtocol(const char * protoPrefix, OpalTransportAddress & addr)
+{
+  OpalTransportAddress compatibleTo("*", 0, protoPrefix);
+  for (OpalListenerList::iterator listener = listeners.begin(); listener != listeners.end(); ++listener) {
+    addr = listener->GetLocalAddress();
+    if (addr.IsCompatible(compatibleTo))
+      return true;
+   }
+  return false;
+}
 
 #if OPAL_PTLIB_SSL
 PString OpalEndPoint::GetSSLCertificate() const
@@ -604,4 +652,10 @@ PString OpalEndPoint::GetSSLCertificate() const
 }
 #endif
 
+void OpalEndPoint::AddIMMediaFormats(OpalMediaFormatList & mediaFormats, const OpalConnection * connection) const
+{
+  manager.AddIMMediaFormats(mediaFormats, connection);
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
