@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23294 $
+ * $Revision: 23874 $
  * $Author: rjongbloed $
- * $Date: 2009-08-28 06:26:26 +0000 (Fri, 28 Aug 2009) $
+ * $Date: 2009-12-14 03:45:30 +0000 (Mon, 14 Dec 2009) $
  */
 
 #include <ptlib.h>
@@ -1653,7 +1653,7 @@ bool OpalWAVRecordManager::Mixer_T::Open(const PFilePath & fn, bool mono)
 
   m_file.SetFormat(OpalWAVFile::fmt_PCM);
   if (!m_file.Open(fn, PFile::ReadWrite, PFile::Create|PFile::Truncate)) {
-    PTRACE(2, "OPAL\tCould not open file \"" << fn << '"');
+    PTRACE(2, "OPAL\tCould not open file \"" << fn << "\": " << m_file.GetErrorText());
     return false;
   }
 
