@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22928 $
+ * $Revision: 23783 $
  * $Author: rjongbloed $
- * $Date: 2009-06-19 05:26:12 +0000 (Fri, 19 Jun 2009) $
+ * $Date: 2009-11-20 06:35:11 +0000 (Fri, 20 Nov 2009) $
  */
 
 #include <ptlib.h>
@@ -141,6 +141,8 @@ SIPHandler::SIPHandler(SIPEndPoint & ep,
 
 SIPHandler::~SIPHandler() 
 {
+  expireTimer.Stop();
+
   if (m_transport) {
     m_transport->CloseWait();
     delete m_transport;
@@ -206,8 +208,16 @@ OpalTransport * SIPHandler::GetTransport()
   if (m_proxy.IsEmpty())
     m_proxy = endpoint.GetProxy();
 
+  SIPURL url;
+  if (!m_proxy.IsEmpty())
+    url = m_proxy;
+  else {
+    url = GetAddressOfRecord();
+    url.AdjustToDNS();
+  }
+
   // Must specify a network interface or get infinite recursion
-  m_transport = endpoint.CreateTransport(m_proxy.IsEmpty() ? GetAddressOfRecord() : m_proxy, "*");
+  m_transport = endpoint.CreateTransport(url, "*");
   return m_transport;
 }
 
@@ -228,7 +238,7 @@ void SIPHandler::SetExpire(int e)
   // if the expire time is more than 20 mins, retry 10mins before expiry
   // if the expire time is less than 20 mins, retry after half of the expiry time
   if (expire > 0 && state < Unsubscribing)
-    expireTimer.SetInterval(0, (unsigned)(expire < 1200*1000 ? expire/2 : expire-600*1000));
+    expireTimer.SetInterval(0, (unsigned)(expire < 20*60 ? expire/2 : expire-10*60));
 }
 
 
@@ -370,10 +380,10 @@ PBoolean SIPHandler::SendRequest(SIPHandler::State newState)
     return true;
   }
 
-  PTRACE(4, "SIP\tRetrying " << GetMethod() << " in " << offlineExpire << " seconds.");
   OnFailed(SIP_PDU::Local_BadTransportAddress);
+
+  PTRACE(4, "SIP\tRetrying " << GetMethod() << " in " << offlineExpire << " seconds.");
   expireTimer.SetInterval(0, offlineExpire); // Keep trying to get it back
-  SetState(Unavailable);
   return true;
 }
 
@@ -384,6 +394,78 @@ PBoolean SIPHandler::OnReceivedNOTIFY(SIP_PDU & /*response*/)
 }
 
 
+void SIPHandler::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & response)
+{
+  // Received a response, so collapse the forking on multiple interfaces.
+
+  transactions.Remove(&transaction); // Take this transaction out of list
+
+  // And kill all the rest
+  PSafePtr<SIPTransaction> transToGo;
+  while ((transToGo = transactions.GetAt(0)) != NULL) {
+    transactions.Remove(transToGo);
+    transToGo->Abort();
+  }
+
+  // Finally end connect mode on the transport
+  m_transport->SetInterface(transaction.GetInterface());
+
+  switch (response.GetStatusCode()) {
+    case SIP_PDU::Failure_UnAuthorised :
+    case SIP_PDU::Failure_ProxyAuthenticationRequired :
+      OnReceivedAuthenticationRequired(transaction, response);
+      break;
+
+    case SIP_PDU::Failure_IntervalTooBrief :
+      OnReceivedIntervalTooBrief(transaction, response);
+      break;
+
+    case SIP_PDU::Failure_TemporarilyUnavailable:
+      OnReceivedTemporarilyUnavailable(transaction, response);
+      break;
+
+    case SIP_PDU::Failure_RequestTimeout :
+      OnTransactionFailed(transaction);
+      break;
+
+    default :
+      switch (response.GetStatusCode()/100) {
+        case 1 :
+          // Do nothing on 1xx
+          break;
+
+        case 2 :
+          OnReceivedOK(transaction, response);
+          break;
+
+        default :
+          OnFailed(response.GetStatusCode());
+      }
+  }
+}
+
+
+void SIPHandler::OnReceivedIntervalTooBrief(SIPTransaction & /*transaction*/, SIP_PDU & response)
+{
+  SetExpire(response.GetMIME().GetMinExpires());
+
+  // Restart the transaction with new authentication handler
+  State oldState = state;
+  state = Unavailable;
+  SendRequest(oldState);
+}
+
+
+void SIPHandler::OnReceivedTemporarilyUnavailable(SIPTransaction & /*transaction*/, SIP_PDU & response)
+{
+  OnFailed(SIP_PDU::Failure_TemporarilyUnavailable);
+
+  unsigned retryAfter = response.GetMIME().GetInteger("Retry-After", offlineExpire);
+  PTRACE(4, "SIP\tRetrying " << GetMethod() << " in " << retryAfter << " seconds.");
+  expireTimer.SetInterval(0, retryAfter); // Have another go in a little bit
+}
+
+
 void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction, SIP_PDU & response)
 {
   bool isProxy = response.GetStatusCode() == SIP_PDU::Failure_ProxyAuthenticationRequired;
@@ -429,6 +511,16 @@ void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction,
       username = proxy.GetUserName();
       password = proxy.GetPassword();
     }
+    else {
+      delete newAuth;
+      PTRACE(1, "SIP\tAuthentication not possible yet.");
+      OnFailed(SIP_PDU::Failure_TemporarilyUnavailable);
+      if (expire > 0 && !transaction.IsCanceled()) {
+        PTRACE(4, "SIP\tRetrying " << GetMethod() << " in " << offlineExpire << " seconds.");
+        expireTimer.SetInterval(0, offlineExpire); // Keep trying to get it back
+      }
+      return;
+    }
   }
 
   newAuth->SetUsername(username);
@@ -449,9 +541,6 @@ void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction,
   m_username = username;
   m_password = password;
 
-  // And end connect mode on the transport
-  CollapseFork(transaction);
-
   // Restart the transaction with new authentication handler
   State oldState = state;
   state = Unavailable;
@@ -459,7 +548,7 @@ void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction,
 }
 
 
-void SIPHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
+void SIPHandler::OnReceivedOK(SIPTransaction & /*transaction*/, SIP_PDU & response)
 {
   response.GetMIME().GetProductInfo(m_productInfo);
 
@@ -483,25 +572,6 @@ void SIPHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
 
   // reset the number of unsuccesful authentication attempts
   authenticationAttempts = 0;
-
-  CollapseFork(transaction);
-}
-
-
-void SIPHandler::CollapseFork(SIPTransaction & transaction)
-{
-  // Take this transaction out
-  transactions.Remove(&transaction);
-
-  // And kill all the rest
-  PSafePtr<SIPTransaction> transToGo;
-  while ((transToGo = transactions.GetAt(0)) != NULL) {
-    transactions.Remove(transToGo);
-    transToGo->Abort();
-  }
-
-  // Finally end connect mode on the transport
-  m_transport->SetInterface(transaction.GetInterface());
 }
 
 
@@ -521,13 +591,11 @@ void SIPHandler::OnTransactionFailed(SIPTransaction & transaction)
 void SIPHandler::OnFailed(SIP_PDU::StatusCodes code)
 {
   switch (code) {
-    case SIP_PDU::Failure_UnAuthorised :
-    case SIP_PDU::Failure_ProxyAuthenticationRequired :
-      break;
-
     case SIP_PDU::Local_TransportError :
+    case SIP_PDU::Local_Timeout :
     case SIP_PDU::Failure_RequestTimeout :
     case SIP_PDU::Local_BadTransportAddress :
+    case SIP_PDU::Failure_TemporarilyUnavailable:
       SetState(Unavailable);
       break;
 
@@ -538,7 +606,6 @@ void SIPHandler::OnFailed(SIP_PDU::StatusCodes code)
       SetState(Unsubscribed);
       ShutDown();
   }
-
 }
 
 
@@ -608,24 +675,39 @@ SIPTransaction * SIPRegisterHandler::CreateTransaction(OpalTransport & trans)
 
   if (expire == 0 || GetState() == Unsubscribing) {
     params.m_expire = 0;
-    params.m_contactAddress = "*";
+    if (params.m_contactAddress.IsEmpty())
+      params.m_contactAddress = "*";
   }
   else {
     params.m_expire = expire;
 
     if (params.m_contactAddress.IsEmpty()) {
-      // Nothing explicit, put in all the bound interfaces.
-      unsigned qvalue = 1000;
       PString userName = SIPURL(params.m_addressOfRecord).GetUserName();
       OpalTransportAddressArray interfaces = endpoint.GetInterfaceAddresses(true, &trans);
-      for (PINDEX i = 0; i < interfaces.GetSize(); ++i) {
-        if (!params.m_contactAddress.IsEmpty())
-          params.m_contactAddress += ", ";
-        SIPURL contact(userName, interfaces[i]);
+      if (params.m_compatibility == SIPRegister::e_CannotRegisterMultipleContacts) {
+        // If translated by STUN then only the external address of the interface is used.
+        SIPURL contact(userName, interfaces[0]);
         contact.Sanitise(SIPURL::ContactURI);
         params.m_contactAddress += contact.AsQuotedString();
-        params.m_contactAddress.sprintf(qvalue < 1000 ? ";q=0.%03u" : ";q=1", qvalue);
-        qvalue -= 1000/interfaces.GetSize();
+      }
+      else {
+        OpalTransportAddress localAddress = trans.GetLocalAddress();
+        unsigned qvalue = 1000;
+        for (PINDEX i = 0; i < interfaces.GetSize(); ++i) {
+          /* If fully compliant, put into the contact field all the bound
+             interfaces. If special then we only put into the contact
+             listeners that are on the same interface. If translated by STUN
+             then only the external address of the interface is used. */
+          if (params.m_compatibility != SIPRegister::e_CannotRegisterPrivateContacts || localAddress.IsEquivalent(interfaces[i], true)) {
+            if (!params.m_contactAddress.IsEmpty())
+              params.m_contactAddress += ", ";
+            SIPURL contact(userName, interfaces[i]);
+            contact.Sanitise(SIPURL::ContactURI);
+            params.m_contactAddress += contact.AsQuotedString();
+            params.m_contactAddress.sprintf(qvalue < 1000 ? ";q=0.%03u" : ";q=1", qvalue);
+            qvalue -= 1000/interfaces.GetSize();
+          }
+        }
       }
     }
     else {
@@ -650,6 +732,8 @@ void SIPRegisterHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & re
   transaction.GetMIME().GetContacts(requestContacts);
   response.GetMIME().GetContacts(replyContacts);
 
+  m_parameters.m_contactAddress.MakeEmpty();
+
   for (std::list<SIPURL>::iterator request = requestContacts.begin(); request != requestContacts.end(); ++request) {
     for (std::list<SIPURL>::iterator reply = replyContacts.begin(); reply != replyContacts.end(); ++reply) {
       if (*request == *reply) {
@@ -658,6 +742,10 @@ void SIPRegisterHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & re
           SetExpire(response.GetMIME().GetExpires(endpoint.GetRegistrarTimeToLive().GetSeconds()));
         else
           SetExpire(expires.AsUnsigned());
+
+        if (!m_parameters.m_contactAddress.IsEmpty())
+          m_parameters.m_contactAddress += ", ";
+        m_parameters.m_contactAddress += request->AsString();
       }
     }
   }
@@ -723,14 +811,16 @@ void SIPRegisterHandler::SendStatus(SIP_PDU::StatusCodes code, State state)
 void SIPRegisterHandler::UpdateParameters(const SIPRegister::Params & params)
 {
   if (!params.m_authID.IsEmpty())
-    m_username = params.m_authID;   // Adjust the authUser if required 
+    m_username = m_parameters.m_authID = params.m_authID;   // Adjust the authUser if required 
   if (!params.m_realm.IsEmpty())
-    m_realm = params.m_realm;   // Adjust the realm if required 
+    m_realm = m_parameters.m_realm = params.m_realm;   // Adjust the realm if required 
   if (!params.m_password.IsEmpty())
-    m_password = params.m_password; // Adjust the password if required 
+    m_password = m_parameters.m_password = params.m_password; // Adjust the password if required 
 
   if (params.m_expire > 0)
-    SetExpire(params.m_expire);
+    SetExpire(m_parameters.m_expire = params.m_expire);
+
+  m_parameters.m_contactAddress = params.m_contactAddress;
 }
 
 
@@ -758,6 +848,10 @@ SIPSubscribeHandler::SIPSubscribeHandler(SIPEndPoint & endpoint, const SIPSubscr
   m_username = params.m_authID;
   m_password = params.m_password;
   m_realm    = params.m_realm;
+
+  // having an agent is the same as having a proxy....near enough
+  if (!params.m_agentAddress.IsEmpty())
+    m_proxy = params.m_agentAddress;
 }
 
 
@@ -1402,7 +1496,7 @@ SIPPublishHandler::~SIPPublishHandler()
 
 SIPTransaction * SIPPublishHandler::CreateTransaction(OpalTransport & t)
 {
-  SetExpire(originalExpire);
+  m_parameters.m_expire = expire;
   return new SIPPublish(endpoint,
                         t, 
                         GetCallID(),
@@ -1590,8 +1684,8 @@ PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByCallID(const PString & cal
  */
 PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByAuthRealm (const PString & authRealm, const PString & userName, PSafetyMode mode)
 {
-  PIPSocket::Address handlerRealmAddress;
-  PIPSocket::Address authRealmAddress(authRealm);
+  PIPSocket::Address handlerRealmAddress, authRealmAddress;
+  PIPSocket::GetHostAddress(authRealm, authRealmAddress);
 
   // if username is specified, look for exact matches
   if (!userName.IsEmpty()) {
@@ -1600,32 +1694,50 @@ PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByAuthRealm (const PString &
     for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
       if ( handler->GetUsername() == userName &&
           (handler->GetRealm().IsEmpty() || handler->GetRealm() == authRealm) &&
-           handler.SetSafetyMode(mode))
+           handler.SetSafetyMode(mode)) {
+        PTRACE(4, "SIP\tLocated existing credentials for ID \"" << userName << "\" at realm \"" << authRealm << '"');
         return handler;
+      }
+    }
+
+    // look for a match to exact AOR name and realm
+    for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+      if ( handler->GetAddressOfRecord().GetUserName() == userName &&
+          (handler->GetRealm().IsEmpty() || handler->GetRealm() == authRealm) &&
+           handler.SetSafetyMode(mode)) {
+        PTRACE(4, "SIP\tLocated existing credentials for AOR user \"" << userName << "\" at realm \"" << authRealm << '"');
+        return handler;
+      }
     }
 
     // look for a match to exact username and realm as hostname
     for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
-      if (PIPSocket::GetHostAddress(handler->GetRealm(), handlerRealmAddress) &&
-          handlerRealmAddress == authRealmAddress &&
-          handler->GetUsername() == userName &&
-          handler.SetSafetyMode(mode))
+      if (userName == handler->GetUsername() &&
+          PIPSocket::GetHostAddress(handler->GetRealm(), handlerRealmAddress) &&
+          handlerRealmAddress  == authRealmAddress &&
+          handler.SetSafetyMode(mode)) {
+        PTRACE(4, "SIP\tLocated existing credentials for ID \"" << userName << "\" at host/address \"" << authRealm << '"');
         return handler;
+      }
     }
   }
 
   // look for a match to exact realm
   for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
-    if (handler->GetRealm() == authRealm && handler.SetSafetyMode(mode))
+    if (handler->GetRealm() == authRealm && handler.SetSafetyMode(mode)) {
+      PTRACE(4, "SIP\tLocated existing credentials for realm \"" << authRealm << '"');
       return handler;
+    }
   }
 
   // look for a match to exact realm as hostname
   for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
     if (PIPSocket::GetHostAddress(handler->GetRealm(), handlerRealmAddress) &&
         handlerRealmAddress == authRealmAddress &&
-        handler.SetSafetyMode(mode))
+        handler.SetSafetyMode(mode)) {
+      PTRACE(4, "SIP\tLocated existing credentials for host/address \"" << authRealm << '"');
       return handler;
+    }
   }
   return NULL;
 }
