@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23347 $
- * $Author: dsandras $
- * $Date: 2009-09-05 15:16:49 +0000 (Sat, 05 Sep 2009) $
+ * $Revision: 23991 $
+ * $Author: rjongbloed $
+ * $Date: 2010-01-29 08:44:30 +0000 (Fri, 29 Jan 2010) $
  */
 
 #include <ptlib.h>
@@ -383,7 +383,8 @@ PBoolean SIPEndPoint::MakeConnection(OpalCall & call,
 
 void SIPEndPoint::OnReleased(OpalConnection & connection)
 {
-  m_receivedConnectionTokens.RemoveAt(connection.GetIdentifier());
+  if (m_receivedConnectionTokens(connection.GetIdentifier()) == connection.GetToken())
+    m_receivedConnectionTokens.RemoveAt(connection.GetIdentifier());
   OpalEndPoint::OnReleased(connection);
 }
 
@@ -525,8 +526,26 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
       if (toToken.IsEmpty()) {
         token = m_receivedConnectionTokens(mime.GetCallID());
         if (!token.IsEmpty()) {
-          m_connectionThreadPool.AddWork(new SIP_PDU_Work(*this, token, pdu), token);
-          return true;
+          PSafePtr<SIPConnection> connection = GetSIPConnectionWithLock(token, PSafeReference);
+          if (connection != NULL) {
+            switch (connection->CheckINVITE(*pdu)) {
+              case SIPConnection::IsNewINVITE : // Process new INVITE
+                break;
+
+              case SIPConnection::IsDuplicateINVITE : // Completely ignore duplicate INVITE
+                return true;
+
+              case SIPConnection::IsReINVITE : // Pass on to worker thread if re-INVITE
+                m_connectionThreadPool.AddWork(new SIP_PDU_Work(*this, token, pdu), token);
+                return true;
+
+              case SIPConnection::IsLoopedINVITE : // Send back error if looped INVITE
+                SIP_PDU response(*pdu, SIP_PDU::Failure_LoopDetected);
+                response.GetMIME().SetProductInfo(GetUserAgent(), connection->GetProductInfo());
+                pdu->SendResponse(transport, response);
+                return true;
+            }
+          }
         }
 
         pdu->SendResponse(transport, SIP_PDU::Information_Trying, this);
