@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23075 $
+ * $Revision: 23767 $
  * $Author: rjongbloed $
- * $Date: 2009-07-05 05:39:52 +0000 (Sun, 05 Jul 2009) $
+ * $Date: 2009-11-13 05:45:45 +0000 (Fri, 13 Nov 2009) $
  */
 
 #include <ptlib.h>
@@ -285,43 +285,104 @@ void SIPURL::ParseAsAddress(const PString & name, const OpalTransportAddress & a
 
 PBoolean SIPURL::InternalParse(const char * cstr, const char * p_defaultScheme)
 {
-  PString defaultScheme = p_defaultScheme != NULL ? p_defaultScheme : "sip";
+  /* This will try to parse an SIP URI according to the RFC3261 EBNF
 
-  displayName = PString::Empty();
-  fieldParameters = PString::Empty();
+        Contact        =  ("Contact" / "m" ) HCOLON
+                          ( STAR / (contact-param *(COMMA contact-param)))
+        contact-param  =  (name-addr / addr-spec) *(SEMI contact-params)
 
+        From           =  ( "From" / "f" ) HCOLON from-spec
+        from-spec      =  ( name-addr / addr-spec ) *( SEMI from-param )
+
+        Record-Route   =  "Record-Route" HCOLON rec-route *(COMMA rec-route)
+        rec-route      =  name-addr *( SEMI rr-param )
+
+        Reply-To       =  "Reply-To" HCOLON rplyto-spec
+        rplyto-spec    =  ( name-addr / addr-spec ) *( SEMI rplyto-param )
+
+        Route          =  "Route" HCOLON route-param *(COMMA route-param)
+        route-param    =  name-addr *( SEMI rr-param )
+
+        To             =  ( "To" / "t" ) HCOLON
+                          ( name-addr / addr-spec ) *( SEMI to-param )
+
+
+     For all of the above the field parameters (contact-params, to-param etc)
+     all degenerate into special cases of the generic-param production. The
+     generic-param is always included as an option for all these fields as
+     well, so for general parsing we just need to parse the EBNF:
+
+        generic-param  =  token [ EQUAL gen-value ]
+        gen-value      =  token / host / quoted-string
+
+     The URI is then embedded in:
+
+        name-addr      =  [ display-name ] LAQUOT addr-spec RAQUOT
+        addr-spec      =  SIP-URI / SIPS-URI / absoluteURI
+        display-name   =  *(token LWS) / quoted-string
+
+        token          =  1*(alphanum / "-" / "." / "!" / "%" / "*"
+                             / "_" / "+" / "`" / "'" / "~" )
+
+        quoted-string  =  SWS DQUOTE *(qdtext / quoted-pair ) DQUOTE
+        qdtext         =  LWS / %x21 / %x23-5B / %x5D-7E
+                                / UTF8-NONASCII
+        quoted-pair    =  "\" (%x00-09 / %x0B-0C / %x0E-7F)
+
+
+     So, what all that means is we distinguish between name-addr and addr-spec
+     by the '<' character, but we can't just search for it if there is a
+     quoted-string before it in the optional display-name field. So check for
+     that and remove it first, then look for the the '<' which tells us if
+     there COULD be a generic-param later.
+   */
+
+  displayName.MakeEmpty();
+  fieldParameters.MakeEmpty();
+
+  while (isspace(*cstr))
+    cstr++;
   PString str = cstr;
 
+  PINDEX endQuote = 0;
+  if (str[0] == '"') {
+    do {
+      endQuote = str.Find('"', endQuote+1);
+      if (endQuote == P_MAX_INDEX) {
+        PTRACE(1, "SIP\tNo closing double quote in URI: " << str);
+        return false; // No close quote!
+      }
+    } while (str[endQuote-1] == '\\');
+
+    displayName = str(1, endQuote-1);
+
+    PINDEX backslash;
+    while ((backslash = displayName.Find('\\')) != P_MAX_INDEX)
+      displayName.Delete(backslash, 1);
+  }
+
   // see if URL is just a URI or it contains a display address as well
-  PINDEX startBracket = str.FindLast('<');
+  PINDEX startBracket = str.Find('<', endQuote);
   PINDEX endBracket = str.Find('>', startBracket);
 
+  const char * defaultScheme = p_defaultScheme != NULL ? p_defaultScheme : "sip";
+
   // see if URL is just a URI or it contains a display address as well
   if (startBracket == P_MAX_INDEX || endBracket == P_MAX_INDEX) {
     if (!PURL::InternalParse(cstr, defaultScheme))
       return false;
-    }
+  }
   else {
     // get the URI from between the angle brackets
     if (!PURL::InternalParse(str(startBracket+1, endBracket-1), defaultScheme))
-      return PFalse;
+      return false;
 
     fieldParameters = str.Mid(endBracket+1).Trim();
 
-    // See if display address quoted
-    PINDEX endQuote = str.FindLast('"', startBracket);
-    PINDEX startQuote = str.Find('"');
-    if (startQuote == P_MAX_INDEX || endQuote == P_MAX_INDEX || startQuote >= endQuote) {
-      // There are no double quotes around the display name, so take
-      // everything before the start angle bracket
+    if (endQuote == 0) {
+      // There were no double quotes around the display name, take
+      // everything before the start angle bracket, sans whitespace
       displayName = str.Left(startBracket).Trim();
-      }
-    else {
-      // Trim quotes off
-      displayName = str(startQuote+1, endQuote-1);
-      PINDEX backslash;
-      while ((backslash = displayName.Find('\\')) != P_MAX_INDEX)
-        displayName.Delete(backslash, 1);
     }
     }
 
@@ -584,9 +645,10 @@ PBoolean SIPMIMEInfo::IsContentLengthPresent() const
 }
 
 
-PCaselessString SIPMIMEInfo::GetContentType() const
+PCaselessString SIPMIMEInfo::GetContentType(bool includeParameters) const
 {
-  return GetString("Content-Type");
+  PCaselessString str = GetString("Content-Type");
+  return str.Left(includeParameters ? P_MAX_INDEX : str.Find(';')).Trim();
 }
 
 
@@ -1008,7 +1070,7 @@ void SIPMIMEInfo::SetEvent(const PString & v)
 }
 
 
-PString SIPMIMEInfo::GetSubscriptionState() const
+PCaselessString SIPMIMEInfo::GetSubscriptionState() const
 {
   return GetString("Subscription-State");       // no compact form
 }
@@ -1046,7 +1108,7 @@ void SIPMIMEInfo::SetOrganization(const PString & v)
 
 static const char UserAgentTokenChars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-.!%*_+`'~";
 
-void SIPMIMEInfo::GetProductInfo(OpalProductInfo & info)
+void SIPMIMEInfo::GetProductInfo(OpalProductInfo & info) const
 {
   PCaselessString str = GetUserAgent();
   if (str.IsEmpty()) {
@@ -1398,12 +1460,18 @@ PObject::Comparison SIPDigestAuthentication::Compare(const PObject & other) cons
   if (otherAuth == NULL)
     return LessThan;
 
-  Comparison result = GetAuthRealm().Compare(otherAuth->GetAuthRealm());
+  if (algorithm < otherAuth->algorithm)
+    return LessThan;
+  if (algorithm > otherAuth->algorithm)
+    return GreaterThan;
+
+  Comparison result = authRealm.Compare(otherAuth->authRealm);
   if (result != EqualTo)
     return result;
 
-  if (GetAlgorithm() != otherAuth->GetAlgorithm())
-    return GetAlgorithm() < otherAuth->GetAlgorithm() ? LessThan : GreaterThan;
+  result = nonce.Compare(otherAuth->nonce);
+  if (result != EqualTo)
+    return result;
 
   return SIPAuthentication::Compare(other);
 }
@@ -1471,7 +1539,7 @@ PBoolean SIPDigestAuthentication::Parse(const PString & _auth, PBoolean proxy)
 
 PBoolean SIPDigestAuthentication::Authorise(SIP_PDU & pdu) const
 {
-  PTRACE(3, "SIP\tAdding authentication information");
+  PTRACE(3, "SIP\tAdding authentication information for user \"" << username << "\" at realm \"" << authRealm << '"');
 
   PMessageDigest5 digestor;
   PMessageDigest5::Code a1, a2, entityBodyCode, response;
@@ -1836,7 +1904,7 @@ void SIP_PDU::Construct(Methods meth,
             dialog.GetRemoteURI().AsQuotedString(),
             dialog.GetLocalURI().AsQuotedString(),
             dialog.GetCallID(),
-            dialog.GetNextCSeq(),
+            dialog.GetNextCSeq(dialog.IsEstablished() && meth != Method_ACK ? 1 : 0),
             via.GetHostAddress());
 
   SetRoute(dialog.GetRouteSet()); // Possibly adjust the URI and the route
@@ -2341,7 +2409,7 @@ void SIP_PDU::SetSDP(SDPSessionDescription * sdp)
 
 SIPDialogContext::SIPDialogContext()
   : m_callId(SIPTransaction::GenerateCallID())
-  , m_lastSentCSeq(0)
+  , m_lastSentCSeq(1)
   , m_lastReceivedCSeq(0)
 {
 }
@@ -2531,16 +2599,16 @@ SIPTransaction::~SIPTransaction()
 
 PBoolean SIPTransaction::Start()
 {
-  if (state == Completed)
+    if (state == Completed)
     return PTrue;
 
-  if (connection != NULL)
-    connection->OnStartTransaction(*this);
+    if (connection != NULL)
+      connection->OnStartTransaction(*this);
 
-  endpoint.AddTransaction(this);
+    endpoint.AddTransaction(this);
 
-  if (state != NotStarted) {
-    PAssertAlways(PLogicError);
+    if (state != NotStarted) {
+      PAssertAlways(PLogicError);
     return PFalse;
   }
 
@@ -2846,7 +2914,7 @@ void SIPTransaction::SetTerminated(States newState)
     switch (state) {
       case Terminated_Timeout :
       case Terminated_RetriesExceeded:
-        statusCode = SIP_PDU::Failure_RequestTimeout;
+        statusCode = SIP_PDU::Local_Timeout;
         break;
 
       case Terminated_TransportError :
@@ -2935,6 +3003,7 @@ SIPRegister::Params::Params()
   , m_minRetryTime(PMaxTimeInterval)
   , m_maxRetryTime(PMaxTimeInterval)
   , m_userData(NULL)
+  , m_compatibility(SIPRegister::e_FullyCompliant)
 {
 }
 
@@ -3009,6 +3078,12 @@ PObject::Comparison SIPSubscribe::EventPackage::InternalCompare(PINDEX offset, P
   if (myIdPtr == NULL && theirIdPtr == NULL)
     return EqualTo;
 
+  if (myIdPtr == NULL)
+    return LessThan;
+
+  if (theirIdPtr == NULL)
+    return GreaterThan;
+
   const char * myIdEnd = strchr(myIdPtr, ';');
   PINDEX myIdLen = myIdEnd != NULL ? myIdEnd - myIdPtr : strlen(myIdPtr);
 
@@ -3176,14 +3251,12 @@ void SIPRefer::Construct(SIPConnection & connection, OpalTransport & /*transport
 SIPReferNotify::SIPReferNotify(SIPConnection & connection, OpalTransport & transport, StatusCodes code)
   : SIPTransaction(connection, transport, Method_NOTIFY)
 {
-  PStringStream str;
-  
   mime.SetProductInfo(connection.GetEndPoint().GetUserAgent(), connection.GetProductInfo());
-  mime.SetSubscriptionState("terminated;reason=noresource"); // Do not keep an internal state
+  mime.SetSubscriptionState(code < Successful_OK ? "active" : "terminated;reason=noresource");
   mime.SetEvent("refer");
-  mime.SetContentType("message/sipfrag;version=2.0");
-
+  mime.SetContentType("message/sipfrag");
 
+  PStringStream str;
   str << "SIP/" << versionMajor << '.' << versionMinor << " " << code << " " << GetStatusCodeDescription(code);
   entityBody = str;
 }
