@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 22640 $
+ * $Revision: 23782 $
  * $Author: rjongbloed $
- * $Date: 2009-05-19 03:19:20 +0000 (Tue, 19 May 2009) $
+ * $Date: 2009-11-20 05:59:13 +0000 (Fri, 20 Nov 2009) $
  */
 
 #include <ptlib.h>
@@ -251,7 +251,7 @@ OpalFaxMediaStream::OpalFaxMediaStream(OpalFaxConnection & conn,
   , m_receive(receive)
   , m_stationId(stationId)
 {
-  SetDataSize(RTP_DataFrame::MaxMtuPayloadSize);
+  SetDataSize(RTP_DataFrame::MaxMtuPayloadSize, 1);
 }
 
 PBoolean OpalFaxMediaStream::Open()
@@ -347,6 +347,8 @@ PBoolean OpalFaxMediaStream::ReadPacket(RTP_DataFrame & packet)
     PINDEX len = m_faxCallInfo->socket.GetLastReadCount();
     packet.SetPayloadType(RTP_DataFrame::MaxPayloadType);
     packet.SetPayloadSize(len);
+    packet.SetTimestamp(timestamp);
+    timestamp += len/2;
 
 #if WRITE_PCM_FILE
     static int file = _open("t38_audio_in.pcm", _O_BINARY | _O_CREAT | _O_TRUNC | _O_WRONLY, _S_IREAD | _S_IWRITE);
@@ -469,11 +471,6 @@ PBoolean OpalFaxMediaStream::Close()
     faxCallInfo->socket.Close();
     faxCallInfo->spanDSP.Close();
 
-    if (faxCallInfo->stdoutThread != NULL) {
-      delete faxCallInfo->stdoutThread;
-      faxCallInfo->stdoutThread = NULL;
-    }
-
     PWaitAndSignal m2(faxMapMutex);
 
     OpalFaxCallInfoMap_T::iterator r = faxCallInfoMap.find(sessionToken);
@@ -501,6 +498,11 @@ PBoolean OpalFaxMediaStream::Close()
     faxCallInfoMap.erase(sessionToken);
   }
 
+  if (faxCallInfo->stdoutThread != NULL) {
+    delete faxCallInfo->stdoutThread;
+    faxCallInfo->stdoutThread = NULL;
+  }
+
   // delete the object
   delete faxCallInfo;
 
@@ -515,11 +517,13 @@ PBoolean OpalFaxMediaStream::IsSynchronous() const
 }
 
 
+#if OPAL_STATISTICS
 void OpalFaxMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromPatch) const
 {
   OpalMediaStream::GetStatistics(statistics, fromPatch);
   statistics.m_fax = m_statistics;
 }
+#endif
 
 
 static bool ExtractValue(const PString & msg, PINDEX & position, int & value, char sep = '=')
@@ -549,7 +553,11 @@ void OpalFaxMediaStream::ReadStdOut(PThread &, INT)
     if (c < 0) {
       PTRACE(2, "Fax\tError reading from " << m_faxCallInfo->spanDSP.GetName()
              << ": " << m_faxCallInfo->spanDSP.GetErrorText(PChannel::LastReadError));
+#if OPAL_STATISTICS
       m_connection.OnFaxCompleted(m_statistics.m_result != 0);
+#else
+      m_connection.OnFaxCompleted(m_result != 0);
+#endif
       return;
     }
 
@@ -572,8 +580,9 @@ void OpalFaxMediaStream::ReadStdOut(PThread &, INT)
       continue;
     }
 
-    int result, errorCorrection;
     PINDEX position = 0;
+#if OPAL_STATISTICS
+    int result, errorCorrection;
     if (ExtractValue(msg, position, result) &&
         ExtractValue(msg, position, m_statistics.m_bitRate) &&
         ExtractValue(msg, position, m_statistics.m_compression) &&
@@ -593,6 +602,9 @@ void OpalFaxMediaStream::ReadStdOut(PThread &, INT)
       m_statistics.m_result = result; // Only set this if everything parsed correctly
       m_statistics.m_errorCorrection = errorCorrection != 0;
     }
+#else
+    ExtractValue(msg, position, m_result);
+#endif
     PTRACE(4, "Fax\tSpanDSP Output:\n" << msg);
     notInStats = true;
     msg.MakeEmpty();
